------------------------------------------------------------------------
--! @file AddressDecoder.vhd
--! @brief <b>reduced Adressdecoder for Combinational Loop testing</b><p>
--!     Assign oBlock_6 within that for loop to get a Combinational Loop
--!     at sAccessViolationWr.
--!     Comment out that any of the for loops to avoid the Combinational 
--!     Loop Synthesis warning
--!
--! </p>
--! <p>
--! <b>Source:</b><p>
------------------------------------------------------------------------

library IEEE;
 use IEEE.std_logic_1164.all;
 use ieee.numeric_std.all;

library work;
 use work.SigPathDefines.all;

entity AddressDecoder is
    generic (DRW: integer := 15                                         --! data register width (Default)
             );
    port
    (
        iReset                  : in    std_logic;                      --! Reset
        iSysClk                 : in    std_logic;                      --! Clock
        -- DSP Interface
        iDspAddrA               : in    std_logic_vector(ABW downto 0); --! DSP Address
        ioDspDataD              : inout std_logic_vector(DRW downto 0); --! DSP Databus
        iDspCsN                 : in    std_logic;                      --! DSP CS signal
        iDspRwN                 : in    std_logic;                      --! DSP Read/WriteN

        oAccessViolation        : out   std_logic;                      --! Signalize a access violation (invalid address) in register 'SystemInfo'
       
        oBlock_1                : out tSP_Words;
        oBlock_2                : out tSP_Words;
        oBlock_3                : out tSP_Words;
        oBlock_4                : out tSP_Words;
        oBlock_5                : out tSP_Words;
        oBlock_6                : out tSP_Words
    );
end AddressDecoder;

architecture RTL of AddressDecoder is

    constant cAllZero           : std_logic_vector(DRW downto 0) := x"0000";
    signal sDspAddrA            : unsigned(ABW downto 0);           --! DSP Address signal
    signal sDspDataInD          : std_logic_vector(DRW downto 0);   --! DSP Databus Input signal
    signal sDspCsN              : std_logic;
    signal sDspRwN              : std_logic;
    signal sAccessViolationWr   : std_logic;                        --! Write Access violation
    
    CONSTANT addr_Block1Start           : unsigned(ABW downto 0) := to_unsigned(100, ABW+1);
    CONSTANT addr_Block2Start           : unsigned(ABW downto 0) := to_unsigned(148, ABW+1);
    CONSTANT addr_Block3Start           : unsigned(ABW downto 0) := to_unsigned(196, ABW+1);
    CONSTANT addr_Block4Start           : unsigned(ABW downto 0) := to_unsigned(244, ABW+1);
    CONSTANT addr_Block5Start           : unsigned(ABW downto 0) := to_unsigned(292, ABW+1);
    CONSTANT addr_Block6Start           : unsigned(ABW downto 0) := to_unsigned(340, ABW+1);

    
    
    
begin


  P_SAMPLE_DSP : process (iReset, iSysClk)
  begin
    if iReset = '1' then
      sDspRwN       <= '1';
      sDspCsN       <= '1';
      sDspAddrA     <= (others => '0');
      sDspDataInD   <= (others => '0');
    elsif rising_edge(iSysClk) then
        sDspRwN     <= iDspRwN;
        sDspCsN     <= iDspCsN;
        sDspAddrA   <= unsigned(iDspAddrA);
        sDspDataInD <= ioDspDataD;
    end if;
  end process;



    P_DspWrite : process (iReset, iSysClk)
    begin
        if iReset = '1' then
            sAccessViolationWr  <= '0';
            oBlock_1            <= (others => (others => '0'));
            oBlock_2            <= (others => (others => '0'));
            oBlock_3            <= (others => (others => '0'));
            oBlock_4            <= (others => (others => '0'));
            oBlock_5            <= (others => (others => '0'));
            oBlock_6            <= (others => (others => '0'));

        elsif rising_edge(iSysClk) then
            if sDspCsN = '0' and sDspRwN = '0' then             -- write access
            
                sAccessViolationWr  <= '1';

                -- access blocks
                for i in 0 to BLOCK_SIZE-1 loop
                    if sDspAddrA = addr_Block1Start +  i then
                        oBlock_1(i)           <= sDspDataInD;
                        sAccessViolationWr  <= sAccessViolationWr;  -- keep previous state of sAccessViolationWr
                    end if;
                end loop;

                for i in 0 to BLOCK_SIZE-1 loop
                    if sDspAddrA = addr_Block2Start +  i then
                        oBlock_2(i)           <= sDspDataInD;
                        sAccessViolationWr  <= sAccessViolationWr;  -- keep previous state of sAccessViolationWr
                    end if;
                end loop;
                for i in 0 to BLOCK_SIZE-1 loop
                    if sDspAddrA = addr_Block3Start +  i then
                        oBlock_3(i)           <= sDspDataInD;
                        sAccessViolationWr  <= sAccessViolationWr;  -- keep previous state of sAccessViolationWr
                    end if;
                end loop;
                for i in 0 to BLOCK_SIZE-1 loop
                    if sDspAddrA = addr_Block4Start +  i then
                        oBlock_4(i)           <= sDspDataInD;
                        sAccessViolationWr  <= sAccessViolationWr;  -- keep previous state of sAccessViolationWr
                    end if;
                end loop;
                for i in 0 to BLOCK_SIZE-1 loop
                    if sDspAddrA = addr_Block5Start +  i then
                        oBlock_5(i)           <= sDspDataInD;
                        sAccessViolationWr  <= sAccessViolationWr;  -- keep previous state of sAccessViolationWr
                    end if;
                end loop;
                
    -- UNCOMMENT THE FOLLOWING CODE TO GET A COMBINATIONAL LOOP
                for i in 0 to BLOCK_SIZE-1 loop
                    if sDspAddrA = addr_Block6Start +  i then
                        oBlock_6(i)           <= sDspDataInD;
                        sAccessViolationWr  <= sAccessViolationWr;  -- keep previous state of sAccessViolationWr
                    end if;
                end loop;

                assert sAccessViolationWr = '0' report "case defaulted! AddressDecoder Write Access" severity failure;

            end if; -- if sDspRwN = '0' then                            -- write access
        end if;         -- if rising_edge(iSysClkC)
    end process;


    -- DSP Accessviolation
    oAccessViolation    <= sAccessViolationWr;


end RTL;